using System;
using System.Collections.Generic;
using System.Text;
using System.Collections;
using System.Reflection;

namespace com.tr.tms.abstractionLayer
{
    /// <summary>
    ///  The CalaisModel contains all the extracted and resolved data.
    /// </summary>
    public class CalaisModel
    {
        /// <summary>
        ///  The Dictionary contains all the CalaisObjects from the model.
        /// </summary>
        private Dictionary<String, CalaisObject> classesById = new Dictionary<String, CalaisObject>();//key=id, value=CalaisObject
        private Dictionary<String, List<CalaisObject>> classesByType = new Dictionary<String, List<CalaisObject>>();//key=field name, value=field values



        public void AddCalaisObject(CalaisObject calaisObject){
            classesById.Add(calaisObject.ObjectId, calaisObject);
            String type = calaisObject.Type;
            List<CalaisObject> objectsByType;
            if(!classesByType.TryGetValue(type, out objectsByType)){
                objectsByType = new List<CalaisObject>();
                classesByType.Add(type, objectsByType);
            }
            objectsByType.Add(calaisObject);
        }

        public CalaisObject GetCalaisObjectById(String id)
        {
            if (String.IsNullOrWhiteSpace(id))
            {
                return null;
            }
            CalaisObject calaisObject;
            classesById.TryGetValue(id, out calaisObject);
            return calaisObject;
        }

        public List<CalaisObject> GetCalaisObjectByType(String type)
        {
            if (String.IsNullOrWhiteSpace(type))
            {
                return null;
            }
            List<CalaisObject> returnedValue;
            classesByType.TryGetValue(type, out returnedValue);
            return returnedValue;
        }

        public List<string> GetAllTypes()
        {
            return new List<string>(classesByType.Keys);
        }

        public List<CalaisObject> GetAllCalaisObjects()
        {
            return new List<CalaisObject>(classesById.Values);
        }


    }
}
