using System;
using System.Text;
using System.Reflection;
using System.Collections.Generic;

namespace com.tr.tms.abstractionLayer
{

	/// <summary>
 	/// The Basic class of Calais Model.
 	/// </summary>
	public class CalaisObject
	{
		/// <summary>
 		/// URI of the the object
 		/// </summary>
		private String objectId;
		public String ObjectId 
		{ 
			get{ return objectId; } 
			set{ objectId = value; }
		}
		
		/// <summary>
 		/// The type of the object
        /// for example:"http://.../1/type/em/r/Merger"
 		/// </summary>
		private string type;
		public string Type 
		{ 
			get{ return type; } 
			set{ type = value; }
		}

        /// <summary>
        ///  The Dictionary contains all the literals fields
        /// </summary>
        private Dictionary<String, List<String>> literals = new Dictionary<String, List<String>>();
        public Dictionary<String, List<String>> Literals
        {
            get { return literals; }
        }

    	public void AddLiteral(String fieldName, String value)
        {
            List<String> values ;
            
            if(literals.ContainsKey(fieldName))
            {
                values = literals[fieldName];
            }
            else
            {
                values = new List<String>();
                literals.Add(fieldName, values);
            }
            values.Add(value);
        }

        /// <summary>
        ///  The Dictionary contains all the externalURIs fields
        /// </summary>
        private Dictionary<String, List<String>> externalURIs = new Dictionary<String, List<String>>();
        public Dictionary<String, List<String>> ExternalURIs
        {
            get { return externalURIs; }
        }

        public void AddExternalURI(String fieldName, String value)
        {
            List<String> values;

            if (!externalURIs.TryGetValue(fieldName, out values)) 
            {
                values = new List<String>();
                externalURIs.Add(fieldName, values);
            }
            values.Add(value);
        }

        /// <summary>
        ///  The Dictionary contains all the references objects
        /// </summary>
        private Dictionary<String, List<CalaisObject>> references = new Dictionary<String, List<CalaisObject>>();
        public Dictionary<String, List<CalaisObject>> References
        {
            get { return references; }
        }

        public void AddReference(String fieldName, CalaisObject value)
        {
            List<CalaisObject> values;

            if (!references.TryGetValue(fieldName, out values))            
            {
                values = new List<CalaisObject>();
                references.Add(fieldName, values);
            }
            values.Add(value);
        }

        /// <summary>
        ///  The Dictionary contains all the backReferences objects
        /// </summary>
        private Dictionary<String, Dictionary<String, List<CalaisObject>>> backReferences = new Dictionary<String, Dictionary<String, List<CalaisObject>>>();
        public Dictionary<String, Dictionary<String, List<CalaisObject>>> BackReferences
        {
            get { return backReferences; }
        }

        public void AddBackReference(String fieldName, CalaisObject calaisObject)
        {
            String type = calaisObject.Type;
            Dictionary<String, List<CalaisObject>> typesValues;
            if(!backReferences.TryGetValue(fieldName, out typesValues))
            {      
                typesValues = new Dictionary<String, List<CalaisObject>>();
                backReferences.Add(fieldName, typesValues);
            }
            List<CalaisObject> values;
            if(!typesValues.TryGetValue(type, out values))
            {
                values = new List<CalaisObject>();
                typesValues.Add(type, values);
            }
            values.Add(calaisObject);
        }

        public Dictionary<String, List<CalaisObject>> GetBackReferencesByFieldName(String fieldName)
        {
            Dictionary<String, List<CalaisObject>> returnedValue;
            backReferences.TryGetValue(fieldName, out returnedValue);
            return returnedValue;
        }

        public CalaisObject(String objectId, String type)
		{
            this.objectId = objectId;
            this.type = type;
	    }

		public override string ToString()
        {
            StringBuilder sb = new StringBuilder();
        	sb.AppendFormat("{0}: {1} ", "objectId", this.ObjectId);
            return sb.ToString();
        }

	
	}
}
